// Here macro wrappers for mathematical functions are written. The wrappers are
// coded in misc/mathwrap.c file and can be included or excluded by
// USE_MATH_WRAPPERS macro. The wrappers are needed if the same behaviour is wanted
// under different operating systems. The functions themselves behave differently in
// some cases dealing with special codes (not a number, infinity etc.).
// Author: Janez Puhan

#define USE_MATH_WRAPPERS

#ifndef MATHWRAP_INCLUDDED

#define MATHWRAP_INCLUDDED

#ifdef USE_MATH_WRAPPERS

extern double hugewrap();
#define HUGE hugewrap()

extern double fabsnolimit(double x);
extern double sinnolimit(double x);
extern double cosnolimit(double x);
extern double tannolimit(double x);
extern double sinhnolimit(double x);
extern double coshnolimit(double x);
extern double tanhnolimit(double x);
extern double asinnolimit(double x);
extern double acosnolimit(double x);
extern double atannolimit(double x);
extern double atan2nolimit(double y, double x);
extern double lognolimit(double x);
extern double log10nolimit(double x);
extern double expnolimit(double x);
extern double sqrtnolimit(double x);
extern double pownolimit(double x, double y);
extern double floornolimit(double x);
extern double ceilnolimit(double x);

extern double fabslimit(double x);
extern double sinlimit(double x);
extern double coslimit(double x);
extern double tanlimit(double x);
extern double sinhlimit(double x);
extern double coshlimit(double x);
extern double tanhlimit(double x);
extern double asinlimit(double x);
extern double acoslimit(double x);
extern double atanlimit(double x);
extern double atan2limit(double y, double x);
extern double loglimit(double x);
extern double log10limit(double x);
extern double explimit(double x);
extern double sqrtlimit(double x);
extern double powlimit(double x, double y);
extern double floorlimit(double x);
extern double ceillimit(double x);

extern double (*fabswrap)(double x);
extern double (*sinwrap)(double x);
extern double (*coswrap)(double x);
extern double (*tanwrap)(double x);
extern double (*sinhwrap)(double x);
extern double (*coshwrap)(double x);
extern double (*tanhwrap)(double x);
extern double (*asinwrap)(double x);
extern double (*acoswrap)(double x);
extern double (*atanwrap)(double x);
extern double (*atan2wrap)(double y, double x);
extern double (*logwrap)(double x);
extern double (*log10wrap)(double x);
extern double (*expwrap)(double x);
extern double (*sqrtwrap)(double x);
extern double (*powwrap)(double x, double y);
extern double (*floorwrap)(double x);
extern double (*ceilwrap)(double x);

#define fabs(x) (*fabswrap)(x)
#define sin(x) (*sinwrap)(x)
#define cos(x) (*coswrap)(x)
#define tan(x) (*tanwrap)(x)
#define sinh(x) (*sinhwrap)(x)
#define cosh(x) (*coshwrap)(x)
#define tanh(x) (*tanhwrap)(x)
#define asin(x) (*asinwrap)(x)
#define acos(x) (*acoswrap)(x)
#define atan(x) (*atanwrap)(x)
#define atan2(y, x) (*atan2wrap)(y, x)
#define log(x) (*logwrap)(x)
#define log10(x) (*log10wrap)(x)
#define exp(x) (*expwrap)(x)
#define sqrt(x) (*sqrtwrap)(x)
#define pow(x, y) (*powwrap)(x, y)
#define floor(x) (*floorwrap)(x)
#define ceil(x) (*ceilwrap)(x)

#else	// USE_MATH_WRAPPERS

#include <math.h>

#endif	// USE_MATH_WRAPPERS

#endif	// MATHWRAP_INCLUDDED
